#!/usr/bin/python3
# Script that generate a list of all Ubuntu MD5s
# Copyright (C) 2014 Pietro Albini
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import argparse
import urllib.request
import re

DEFAULT = {'ubuntu': {'parser': 'official', 'releases': ['desktop', 'server'], 'versions': ['latest', 'lts'], 'archs': ['i386', 'amd64']},
           'ita': {'parser': 'ita', 'releases': ['desktop'], 'versions': ['ita'], 'archs': ['i386', 'amd64']}
          }
DERIVATIVES = {'kubuntu': {'parser': 'derivatives', 'releases': ['desktop'], 'versions': ['latest'], 'archs': ['i386', 'amd64']},
               'xubuntu': {'parser': 'derivatives', 'releases': ['desktop'], 'versions': ['latest'], 'archs': ['i386', 'amd64']},
               'lubuntu': {'parser': 'derivatives', 'releases': ['desktop'], 'versions': ['latest'], 'archs': ['i386', 'amd64']},
               'edubuntu': {'parser': 'derivatives', 'releases': ['dvd'], 'versions': ['latest'], 'archs': ['i386', 'amd64']},
               'ubuntustudio': {'parser': 'derivatives', 'releases': ['dvd'], 'versions': ['latest'], 'archs': ['i386', 'amd64']},
               'mythbuntu': {'parser': 'derivatives', 'releases': ['desktop'], 'versions': ['lts'], 'archs': ['i386', 'amd64']}
              }

def get_base_md5_dict(releases, archs):
    """ Build the base md5s dict """
    result = {}
    for release in releases:
        sub = {}
        for arch in archs:
            sub[arch] = None
        result[release] = sub
    return result

def official_md5_list(distro, version, releases, archs):
    if args.verbose:
        print('Fetching "http://releases.ubuntu.com/'+version+'/MD5SUMS"...')
    web = urllib.request.urlopen('http://releases.ubuntu.com/'+version+'/MD5SUMS')
    lines = web.read().decode('utf-8').split('\n')
    md5s = get_base_md5_dict(releases, archs)
    for line in lines:
        try:
            info = re.sub(r'(.*) \*ubuntu-'+version+r'-(.*)-(.*)\.iso', r'\1|\2|\3', line).split('|') # Parse the line
            if info[1] in md5s and info[2] in md5s[ info[1] ]:
                md5s[ info[1] ][ info[2] ] = {'md5': info[0], 'filename': 'ubuntu-'+version+'-'+info[1]+'-'+info[2]+'.iso'}
        except IndexError:
            pass
    return md5s

def derivatives_md5_list(distro, version, releases, archs):
    if args.verbose:
        print('Fetching "http://cdimage.ubuntu.com/'+distro+'/releases/'+version+'/release/MD5SUMS"...')
    web = urllib.request.urlopen('http://cdimage.ubuntu.com/'+distro+'/releases/'+version+'/release/MD5SUMS')
    lines = web.read().decode('utf-8').split('\n')
    md5s = get_base_md5_dict(releases, archs)
    for line in lines:
        try:
            info = re.sub(r'(.*) \*'+distro+'-'+version+r'-(.*)-(.*)\.iso', r'\1|\2|\3', line).split('|') # Parse the line
            if info[1] in md5s and info[2] in md5s[ info[1] ]:
                md5s[ info[1] ][ info[2] ] = {'md5': info[0], 'filename': distro+'-'+version+'-'+info[1]+'-'+info[2]+'.iso'}
        except IndexError:
            pass
    return md5s

def italian_md5_list(distro, version, releases, archs):
    md5s = get_base_md5_dict(releases, archs)
    for arch in archs:
        for release in releases:
            if args.verbose:
                print('Fetching "http://release.ubuntu-it.org/'+version+'/'+arch+'/'+version+'-'+release+'-'+arch+'.iso.md5"...')
            web = urllib.request.urlopen('http://release.ubuntu-it.org/'+version+'/'+arch+'/'+version+'-'+release+'-'+arch+'.iso.md5')
            lines = web.read().decode('utf-8').split('\n')
            for line in lines:
                try:
                    info = re.sub(r'(.*)  (.*)-(.*)-(.*)\.iso', r'\1|\2|\3|\4', line).split('|')
                    md5s[ info[2] ][ info[3] ] = {'md5': info[0], 'filename': info[1]+'-'+info[2]+'-'+info[3]+'.iso'}
                except IndexError:
                    pass
    return md5s

def get(distro, info):
    """ Get md5s of a specific distro """
    getters = {'official': official_md5_list, 'derivatives': derivatives_md5_list, 'ita': italian_md5_list}
    versions = set( info['versions'] ) & available # available versions

    result = {}
    for version in versions:
        if args.two_lts and distro == 'mythbuntu':
            version = "latest"
        output = getters[ info['parser'] ]( distro, getattr(args, version), info['releases'], info['archs'] )
        if distro == 'ita':
            version = 'lts_ita'
        result[version] = output

    return result

def render_php(md5s):
    for distro, versions in md5s.items():
        for version, releases in versions.items():
            for release, archs in releases.items():
                for arch, md5 in archs.items():
                    if md5:
                        if distro == 'ita':
                            distro = 'ubuntu'
                        print("$md5s['"+distro+"']['"+version+"']['"+release+"']['"+arch+"'] = '"+md5['md5']+"';")

def render_moin(md5s):
    for distro, versions in md5s.items():
        for version, releases in versions.items():
            for release, archs in releases.items():
                for arch, md5 in archs.items():
                    if md5:
                        if distro == 'ita':
                            distro = 'ubuntu'
                        print("|| "+md5['md5']+" || "+md5['filename']+" ||")
            print()

if __name__ == '__main__':
    # Setup argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("--latest", help="Set which version is the latest")
    parser.add_argument("--lts", help="Set which version is the LTS")
    parser.add_argument("--ita", help="Set which version is the italian remix")
    parser.add_argument("--derivatives", help="Choose if include derivatives", action="store_true")
    parser.add_argument("--two-lts", help="If latest is an lts", action="store_true", dest="two_lts")
    parser.add_argument("--render-as", help="Set how render the result", default='php', choices=('php', 'moin'))
    parser.add_argument("-v", "--verbose", help="Verbose mode", action="store_true", dest="verbose")
    args = parser.parse_args() # Get arguments

    # Get available versions
    available = set()
    if args.latest:
        available.add('latest')
    if args.lts:
        available.add('lts')
    if args.ita:
        available.add('ita')

    total = {}
    total['ubuntu'] = get('ubuntu', DEFAULT['ubuntu'])
    if args.ita:
        total['ita'] = get('ita', DEFAULT['ita'])
    if args.derivatives:
        for which in DERIVATIVES:
            total[which] = get(which, DERIVATIVES[which])

    if args.verbose:
        print('---')

    if args.render_as == 'php':
        render_php(total)
    elif args.render_as == 'moin':
        render_moin(total)